<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the dashboard.
 *
 * @link 		https://entusiastics.com
 * @since 		1.0.0
 *
 * @package 	Entusiastics_Quiz_Game
 * @subpackage 	Entusiastics_Quiz_Game/includes
 */

class Entusiastics_Quiz_Game {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		protected
	 * @var 		Quiz_Game_Loader 		$loader 		Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since 		1.0.0
	 * @access 		protected
	 * @var 		string 			$plugin_name 		The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * Sanitizer for cleaning user input
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      Quiz_Game_Sanitize    $sanitizer    Sanitizes data
	 */
	private $sanitizer;

	/**
	 * The current version of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		protected
	 * @var 		string 			$version 		The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the Dashboard and
	 * the public-facing side of the site.
	 *
	 * @since 		1.0.0
	 */
	public function __construct() {

		$this->plugin_name = 'entusiastics-quiz-game';
		$this->version = '1.0.0';

		define( 'ENTUSIASTICS_QUIZ_GAME_PATH', dirname( __FILE__ ) );
		define( 'ENTUSIASTICS_QUIZ_GAME_RELATIVE_PATH', dirname( plugin_basename( __FILE__ )));
		define( 'ENTUSIASTICS_QUIZ_GAME_URL', plugin_dir_url( __FILE__ ));
		define( 'ENTUSIASTICS_QUIZ_GAME_PLUGIN_PATH', WP_PLUGIN_DIR."/entusiastics-quiz-game/" );

		// start session only on front-end and Entusiastic Quiz admin pages
		if (!session_id() and (strstr(@$_GET['page'], 'entusiasticsquiz') or !is_admin() or (wp_doing_ajax() and !empty($_POST['entusiastic_game_quiz_action']))) ) {
			@session_start();
		}

		$this->load_dependencies();
		$this->set_locale();
		$this->set_database($this->version);
		$this->define_admin_hooks();
		$this->define_public_hooks();
		$this->define_template_hooks();

		$this->define_widget_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Quiz_Game_Loader. Orchestrates the hooks of the plugin.
	 * - Quiz_Game_i18n. Defines internationalization functionality.
	 * - Quiz_Game_Admin. Defines all hooks for the dashboard.
	 * - Quiz_Game_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-loader.php';

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-database.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the Dashboard.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/'.$this->plugin_name.'-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/'.$this->plugin_name.'-public.php';

		/**
		 * The class responsible for defining all actions creating the templates.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/'.$this->plugin_name.'-template-functions.php';

		/**
		 * The class responsible for all global functions.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-global-functions.php';

		/**
		 * The class responsible for defining all actions shared by the Dashboard and public-facing sides.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-shared.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-widget.php';

		/**
		 * The class responsible for sanitizing user input
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-sanitize.php';

		/**
		 * The class responsible for show Tables
		 */
		if( ! class_exists( 'WP_List_Table' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
		}

		/**
		 * Controllers
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/QuizzesController.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/QuizzesTableController.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/CategoriesController.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/CategoriesTableController.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/ProfilesController.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/ProfilesTableController.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/CompletedController.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/CompletedTableController.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/QuestionsController.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/ResultsController.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/ajax.php';	

		/**
		 * Models
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/models/Objective.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/models/Question.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/models/Quiz.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/models/Category.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/models/Result.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/models/Profile.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/models/Choice.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/models/Completed.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/models/UserAnswer.php';
		
		$this->loader = new Entusiastics_Quiz_Game_Loader();
		$this->sanitizer = new Entusiastics_Quiz_Game_Sanitize();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Quiz_Game_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function set_locale() {

		$plugin_i18n = new Entusiastics_Quiz_Game_i18n();
		$plugin_i18n->set_domain( $this->get_plugin_name() );

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}


	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Quiz_Game_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function set_database($version) {

		$plugin_database = new Entusiastics_Quiz_Game_Database();
		$plugin_database->install($this->version);

		$this->loader->add_action( 'database_create', $plugin_database, 'create_plugin_database' );

	}

	/**
	 * Register all of the hooks related to the dashboard functionality
	 * of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Entusiastics_Quiz_Game_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		
		$this->loader->add_filter( 'plugin_action_links_' . ENTUSIASTICS_QUIZ_GAME_FILE, $plugin_admin, 'link_settings' );

		$this->loader->add_action( 'plugin_row_meta', $plugin_admin, 'link_row', 10, 2 );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'add_menu' );

		$this->loader->add_action( 'admin_init', $plugin_admin, 'register_settings' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'register_fields' );

		$this->loader->add_action( 'admin_notices', $plugin_admin, 'display_admin_notices' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'admin_notices_init' );

		$this->loader->add_action( 'admin_enqueue_scripts',$plugin_admin, 'load_media_files' );

		add_action('wp_ajax_entusiastics_finalize_quizz', 'entusiastics_finalize_quizz');
		add_action('wp_ajax_nopriv_entusiastics_finalize_quizz', 'entusiastics_finalize_quizz');
		
		add_action('wp_ajax_entusiastics_move_questions_ajax', 'entusiastics_move_questions_ajax');
		add_action('wp_ajax_nopriv_entusiastics_move_questions_ajax', 'entusiastics_move_questions_ajax');

		add_action('wp_ajax_entusiastics_add_choice_ajax', 'entusiastics_add_choice_ajax');
		add_action('wp_ajax_nopriv_entusiastics_add_choice_ajax', 'entusiastics_add_choice_ajax');

		add_action('wp_ajax_entusiastics_get_current_latest_quizzes_url', 'entusiastics_get_current_latest_quizzes_url');
		add_action('wp_ajax_nopriv_entusiastics_get_current_latest_quizzes_url', 'entusiastics_get_current_latest_quizzes_url');

	} // define_admin_hooks()

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function define_public_hooks() {

		$plugin_public = new Entusiastics_Quiz_Game_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles', $this->get_version(), TRUE );
		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts', $this->get_version(), TRUE );

		$this->loader->add_action('init', $plugin_public, 'register_shortcodes');
		
		add_action('wp_ajax_entusiastics_quiz_game_ajax', 'entusiastics_quiz_game_ajax');
		add_action('wp_ajax_nopriv_entusiastics_quiz_game_ajax', 'entusiastics_quiz_game_ajax');
		
	} // define_public_hooks()

	/**
	 * Register all of the hooks related to the templates.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_template_hooks() {

		$plugin_templates = new Entusiastics_Quiz_Game_Template_Functions( $this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('entusiastics_show_quiz_template', $plugin_templates, 'entusiastics_quiz_template', 10, 2);
		$this->loader->add_action('entusiastics_show_question_template', $plugin_templates, 'entusiastics_question_template', 10, 2);
		$this->loader->add_action('entusiastics_show_video_template', $plugin_templates, 'entusiastics_video_template', 10, 2);
		$this->loader->add_action('entusiastics_show_result_template', $plugin_templates, 'entusiastics_result_template', 10, 2);
		$this->loader->add_action('entusiastics_show_notfound_template', $plugin_templates, 'entusiastics_not_found_template', 10, 2);
		$this->loader->add_action('entusiastics_show_latest_quizzes_template', $plugin_templates, 'entusiastics_latest_quizzes_template', 10, 2);

	} // define_template_hooks()

	/**
	 * Register all of the hooks shared between public-facing and admin functionality
	 * of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function define_shared_hooks() {

		$plugin_shared = new Entusiastics_Quiz_Game_Shared( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'widgets_init', $plugin_shared, 'widgets_init' );
		$this->loader->add_action( 'save_post_quiz', $plugin_shared, 'flush_widget_cache' );
		$this->loader->add_action( 'deleted_post', $plugin_shared, 'flush_widget_cache' );
		$this->loader->add_action( 'switch_theme', $plugin_shared, 'flush_widget_cache' );

	} // define_shared_hooks()


	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since 		1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since 		1.0.0
	 * @return 		string 					The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since 		1.0.0
	 * @return 		Quiz_Game_Loader 		Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since 		1.0.0
	 * @return 		string 					The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	// Option 2

	/**
	 * Register all of the hooks shared between public-facing and admin functionality
	 * of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function define_widget_hooks() {

		$this->loader->add_action( 'widgets_init', $this, 'widgets_init' );
		$this->loader->add_action( 'deleted_post', $this, 'flush_widget_cache' );
		$this->loader->add_action( 'switch_theme', $this, 'flush_widget_cache' );

	}

	/**
	 * Flushes widget cache
	 *
	 * @since 		1.0.0
	 * @access 		public
	 * @param 		int 		$post_id 		The post ID
	 * @return 		void
	 */
	public function flush_widget_cache( $post_id ) {
	}

	/**
	 * Registers widgets with WordPress
	 *
	 * @since 		1.0.0
	 * @access 		public
	 */
	public function widgets_init() {

		register_widget( 'entusiastics_quiz_game_widget' );

	} // widgets_init()

}
